ScriptName MystWeaponSpeedScript Extends ActiveMagicEffect

Actor Subject
Bool UpdateLock = False

Float LeftWeaponSpeed = 0.0
Float RightWeaponSpeed = 0.0

Float AVSpeedLeftValue = 0.0
Float AVSpeedRightValue = 0.0
Bool ModAVSpeedLeft = False
Bool ModAVSpeedRight = False

Float LastModAVLeftWeaponSpeed = 0.0
Float LastModAVRightWeaponSpeed = 0.0

Bool NeedsFixRight = False
Bool NeedsFixLeft = False

GlobalVariable Property WeaponSpeedLimitGlobal Auto ; Default = 2.00
Float WeaponSpeedLimit = 2.00  ; Copied from the Global, stored within the script instance.

Event OnEffectStart(Actor Target, Actor Caster)
	Debug.Trace("Weapon Speed Fix Effect is active on " +Target)
	Subject = Target
	Initialize(Target)
EndEvent

Event OnEffectFinish(Actor Target, Actor Caster) ; The effect is never naturally removed, but the MCM controls the global that enables or disables the effect.
	Debug.Trace(Subject+ " no longer has a Weapon Speed Fix.")
	StopUpdate(Target)
EndEvent

;/Event OnInit() ; Useful on a ReferenceAlias Script.
	If !Subject
		Subject == Game.GetPlayer()
	EndIf
	Initialize()
EndEvent/;

Function StopUpdate(Actor Target)
	GoToState("")
	While UpdateLock
		Utility.Wait(0.1)
	EndWhile
	StopFix()
	Debug.Notification(Subject.GetBaseObject().GetName()+ " no longer has a Weapon Speed Fix.")
EndFunction
	
Function Initialize(Actor Target)
	GoToState("Active")
EndFunction

Event OnUpdate()
EndEvent

Function WeaponSpeedMaintenance()
	WeaponSpeedLimit = WeaponSpeedLimitGlobal.GetValue()
	RightWeaponSpeed = Subject.GetActorValue("WeaponSpeedMult")
	LeftWeaponSpeed = Subject.GetActorValue("LeftWeaponSpeedMult")
	NeedsFixRight = False
	NeedsFixLeft = False

	If RightWeaponSpeed > WeaponSpeedLimit
		AVSpeedRightValue = (WeaponSpeedLimit - RightWeaponSpeed)
		Debug.Trace("MystWeaponSpeedFix: Pre-Fix RightWeaponSpeed as " +RightWeaponSpeed+ " Fix Value at " +AVSpeedRightValue)
		NeedsFixRight = True
	ElseIf RightWeaponSpeed >= 1.00 && RightWeaponSpeed <= WeaponSpeedLimit
		NeedsFixRight = False
	ElseIf RightWeaponSpeed < 1.00
		AVSpeedRightValue = (1.00 - RightWeaponSpeed)
		Debug.Trace("MystWeaponSpeedFix: Pre-Fix RightWeaponSpeed as " +RightWeaponSpeed+ " Fix Value at " +AVSpeedRightValue)
		NeedsFixRight = True
	EndIf
	
	If LeftWeaponSpeed > WeaponSpeedLimit
		AVSpeedLeftValue = (WeaponSpeedLimit - LeftWeaponSpeed)
		Debug.Trace("MystWeaponSpeedFix: Pre-Fix LeftWeaponSpeed as " +LeftWeaponSpeed+ " Fix Value at " +AVSpeedLeftValue)
		NeedsFixLeft = True
	ElseIf LeftWeaponSpeed >= 1.00 && LeftWeaponSpeed <= WeaponSpeedLimit
		NeedsFixLeft = False
	ElseIf LeftWeaponSpeed < 1.00
		AVSpeedLeftValue = (1.00 - LeftWeaponSpeed)
		Debug.Trace("MystWeaponSpeedFix: Pre-Fix LeftWeaponSpeed as " +LeftWeaponSpeed+ " Fix Value at " +AVSpeedLeftValue)
		NeedsFixLeft = True
	EndIf

	FixWeaponSpeed()
	
EndFunction

Function FixWeaponSpeed()	
	If NeedsFixRight
		If !ModAVSpeedRight
			Subject.ModActorValue("WeaponSpeedMult", AVSpeedRightValue)
			LastModAVRightWeaponSpeed = AVSpeedRightValue
			ModAVSpeedRight = True
			Debug.Trace("MystWeaponSpeedFix: Post-Fix RightWeaponSpeed as " +Subject.GetActorValue("WeaponSpeedMult")+ " Fixed by " +LastModAVRightWeaponSpeed)
		Else
			Subject.ModActorValue("WeaponSpeedMult", (-1 * LastModAVRightWeaponSpeed))
			ModAVSpeedRight = False
			Debug.Trace("MystWeaponSpeedFix: Post-Fix RightWeaponSpeed as " +Subject.GetActorValue("WeaponSpeedMult")+ " Removed ModAV " +LastModAVLeftWeaponSpeed)
		EndIf
	EndIf
	If NeedsFixLeft
		If !ModAVSpeedLeft
			Subject.ModActorValue("LeftWeaponSpeedMult", AVSpeedLeftValue)
			LastModAVLeftWeaponSpeed = AVSpeedLeftValue
			ModAVSpeedLeft = True
			Debug.Trace("MystWeaponSpeedFix: Post-Fix LeftWeaponSpeed as " +Subject.GetActorValue("LeftWeaponSpeedMult")+ " Fixed by " +LastModAVLeftWeaponSpeed)
		Else
			Subject.ModActorValue("LeftWeaponSpeedMult", (-1 * LastModAVLeftWeaponSpeed))
			ModAVSpeedLeft = False
			Debug.Trace("MystWeaponSpeedFix: Post-Fix LeftWeaponSpeed as " +Subject.GetActorValue("LeftWeaponSpeedMult")+ " Removed ModAV " +LastModAVLeftWeaponSpeed)
		EndIf
	EndIf
EndFunction

Function StopFix()
	If ModAVSpeedRight
		Subject.ModActorValue("WeaponSpeedMult", (-1 * LastModAVRightWeaponSpeed))
		ModAVSpeedRight = False
		Debug.Trace("MystWeaponSpeedFix: Post-Fix RightWeaponSpeed as " +Subject.GetActorValue("WeaponSpeedMult")+ " Removed ModAV " +LastModAVLeftWeaponSpeed)
	EndIf
	If ModAVSpeedLeft
		Subject.ModActorValue("LeftWeaponSpeedMult", (-1 * LastModAVLeftWeaponSpeed))
		ModAVSpeedLeft = False
		Debug.Trace("MystWeaponSpeedFix: Post-Fix LeftWeaponSpeed as " +Subject.GetActorValue("LeftWeaponSpeedMult")+ " Removed ModAV " +LastModAVLeftWeaponSpeed)
	EndIf
EndFunction

State Active

Event OnBeginState()
	Debug.Notification("Weapon Speed Fix Effect is active on " +Subject.GetBaseObject().GetName())
	RegisterForSingleUpdate(0.0)
EndEvent

Event OnUpdate()
	If !UpdateLock
		UpdateLock = True
		WeaponSpeedMaintenance()
		UpdateLock = False
		RegisterForSingleUpdate(1.0)
	EndIf
EndEvent

EndState
